//
// LPP Rendering pass 2 - building light buffer
//

#extension GL_ARB_texture_rectangle: enable

varying	vec3          pos;
uniform sampler2DRect dataMap;

#define	MAX_DIST	1.5

float	atten ( float d )								// compute distance attenuation factor, 0 -> 1, MAX_DIST -> 0
{
	const	float	offs0 = 1.0;
	const	float	offs1 = 1.0 / (1.0 + MAX_DIST);
	const	float	scale = 0.5 / (offs0 - offs1);
	
	return scale * ( 1.0 / (1.0 + d) - offs1 );
}

void main (void)
{
	vec3	lightPos   = gl_TexCoord [1].xyz / gl_TexCoord [1].w;
	vec3	lightColor = gl_TexCoord [2].xyz;

	vec4	nz = texture2DRect ( dataMap, gl_FragCoord.xy );
	vec3	n  = nz.xyz;
	vec3	pp = pos * nz.w / pos.z;
	float	d  = length    ( lightPos - pp );
	
	if ( d > MAX_DIST )									// too far away
		discard;
	
	vec3	l    = normalize ( lightPos - pp );
	vec3	v    = normalize ( -pp );
	vec3	h    = normalize ( l + v );
	float	at   = atten     ( d );
	float	lum  = dot ( lightColor, vec3 ( 0.3, 0.59, 0.11 ) );
	float	diff = at * max ( 0.1, dot ( l, n ) );
	float	spec = diff * lum * pow ( max ( 0.001, dot ( h, n ) ), 40.0 );
	
	gl_FragColor = vec4 ( diff * lightColor.rgb, spec );
}
